@@ -3,7 +3,10 @@ use std::mem;
 use hecs::{Component, DynamicBundle, Entity, QueryBorrow, Without, World};
 use hecs_schedule::{error::Result, GenericWorld};
 
-use crate::{AncestorIter, BreadthFirstIterator, Child, ChildrenIter, DepthFirstIterator, Parent};
+use crate::{
+    AncestorIter, BreadthFirstIterator, Child, ChildrenIter, DepthFirstIterator, DepthFirstVisitor,
+    Parent,
+};
 
 /// A trait for modifying the worlds hierarchy. Implemented for `hecs::World`>
 pub trait HierarchyMut {
@@ -49,13 +52,20 @@ where
 
     /// Traverses the immediate children of parent. If parent is not a Parent, an empty iterator is
     /// returned.
-    fn children<T: Component>(&self, parent: Entity) -> ChildrenIter<Self, T>;
+    fn children<T: Component>(&self, parent: Entity) -> ChildrenIter<T>;
 
     /// Traverse the tree upwards. Iterator does not include the child itself.
-    fn ancestors<T: Component>(&self, child: Entity) -> AncestorIter<Self, T>;
+    fn ancestors<T: Component>(&self, child: Entity) -> AncestorIter<T>;
 
     /// Traverse the tree depth first. Iterator does not include the child itself.
-    fn descendants_depth_first<T: Component>(&self, root: Entity) -> DepthFirstIterator<Self, T>;
+    fn descendants_depth_first<T: Component>(&self, root: Entity) -> DepthFirstIterator<T>;
+
+    /// Traverse the tree depth first with an acceptance function
+    fn visit<T: Component, F: Fn(&Self, Entity) -> bool + Component>(
+        &self,
+        root: Entity,
+        accept: F,
+    ) -> DepthFirstVisitor<Self, T, F>;
 
     /// Traverse the tree breadth first. Iterator does not include the child itself.
     fn descendants_breadth_first<T: Component>(
@@ -184,7 +194,7 @@ impl<W: GenericWorld> Hierarchy for W {
         Ok(cur)
     }
 
-    fn children<T: Component>(&self, parent: Entity) -> ChildrenIter<Self, T> {
+    fn children<T: Component>(&self, parent: Entity) -> ChildrenIter<T> {
         self.try_get::<Parent<T>>(parent)
             .and_then(|parent| {
                 let first_child = parent.first_child(self)?;
@@ -201,11 +211,11 @@ impl<W: GenericWorld> Hierarchy for W {
             })
     }
 
-    fn ancestors<T: Component>(&self, child: Entity) -> AncestorIter<Self, T> {
+    fn ancestors<T: Component>(&self, child: Entity) -> AncestorIter<T> {
         AncestorIter::new(self, child)
     }
 
-    fn descendants_depth_first<T: Component>(&self, root: Entity) -> DepthFirstIterator<Self, T> {
+    fn descendants_depth_first<T: Component>(&self, root: Entity) -> DepthFirstIterator<T> {
         DepthFirstIterator::new(self, root)
     }
 
@@ -217,6 +227,14 @@ impl<W: GenericWorld> Hierarchy for W {
         BreadthFirstIterator::new(self, root)
     }
 
+    fn visit<T: Component, F: Fn(&Self, Entity) -> bool + Component>(
+        &self,
+        root: Entity,
+        accept: F,
+    ) -> DepthFirstVisitor<Self, T, F> {
+        DepthFirstVisitor::new(self, root, accept)
+    }
+
     fn roots<T: Component>(&self) -> Result<QueryBorrow<Without<Child<T>, &Parent<T>>>> {
         Ok(self.try_query::<&Parent<T>>()?.without::<Child<T>>())
     }
